#ifndef EYES_H
#define EYES_H

#include <iostream>
#include <map>
#include <vector>
#include <QVector>
#include <QElapsedTimer>
#include "extdeque.hpp"
#include "objectdetect.h"
#include "def.h"
#include "eyes_actions_callback.h"
#include "eyes_config.hpp"

namespace eyetrack
{

/*!
** \brief class Eyes
*/
class	Eyes : public ObjectDetect
{
public:
	static const std::string		DEFAULT_CASCADE;
	/*!
	** \brief
	** Temps en ms pris en compte pour valider un clignement
	*/
	static const qint64				BLINK_TIME_IN_MSEC_TO_EXEC;

	/*!
	** \brief Constructeur
	** @param class Callback	: class de callback pour les actions des yeux
	** @param String folderPath	: chemin vers le dossier ressources de l'eye-tracking
	*/
	Eyes(eyes_actions_callback& callback, const QString& folderPath);
	
	virtual ~Eyes();

	/*!
	** \brief
	** Dtermine et retourne la direction regarde
	** Envoie un signal via la class CallBack pour avertir d'un changement de direction (eyes_actions_callback::emit_signal_direction)
	** @return enum Direction : la direction regarde
	*/
	Direction						lookAt();
	/*!
	** \brief
	** Associe l'image en mmoire  la direction prcise en paramtre
	** @param enum Direction dir : la direction  associer  l'image
	** @return Boolean : true si l'enregistrement a russi
	*/
	bool							trainDirection(const Direction dir);
	/*!
	** \brief
	** Charge la configuration pour les yeux de l'utilisateur
	** @return Boolean : true si le chargement a russi
	*/
	bool							load_config();
	
private:
	QVector<eye_s>					mRightEyeTemplates;
	QVector<eye_s>					mLeftEyeTemplates;
	std::vector<std::string>		mCascades;
	eye_s							mLeftEye;
	eye_s							mRightEye;
	extdeque<Direction>				mDirections;
	extdeque<eyes_blink>			mBlinks;
	QElapsedTimer					mStartBlinkingTimer;
	QElapsedTimer					mDirectionTimer;
	eyes_actions_callback&			mCallback;
	QString							mFolderPath;

	/*!
	** \brief
	** Distingue l'oeil droit de l'oeil gauche.
	** Tente de faire la distinction meme si qu'un oeil n'a t dtect
	*/
	void							distinct_eyes();
	/*!
	** \brief
	** Distingue s'il y a clignement des yeux (persistence sur BLINK_TIME_IN_MSEC_TO_EXEC ms)
	** Envoie un signal via la class CallBack si un clignement est valid
	*/
	void							distinct_blinks();
	/*!
	** \brief
	** Insre la direction passe en paramtre  la fin de la queue
	** @param enum Direction dir
	*/
	void							manageDirections(const Direction dir);
};

}

#endif // !EYES_H