#ifndef OBJECTDETECT_H
#define OBJECTDETECT_H

#include <iostream>
#include <vector>
#include <opencv2/opencv.hpp>

namespace eyetrack
{

/*!
** \brief Classe de dtection d'objet
*/
class ObjectDetect
{
public:
	ObjectDetect();
	~ObjectDetect();

	/*!
	** \brief
	** Charge le fichier Cascade Classifier  utiliser pour la dtection
	** @param String cname	: le nom du fichier
	** @return Boolean 		: true si le fichier est correctement charg
	*/
	bool								setClassifier(const std::string& cname);
	/*!
	** \brief
	** Renseigne l'image dans laquelle faire la dtection
	** @param Mat frameToDetectFrom	: l'image
	*/
	void								setFrame(const cv::Mat& frameToDetectFrom);
	/*!
	** \brief
	** Dmarre la dtection
	** @param Integer objLimit			: le nombre maximum d'objets  dtecter, 0 pour aucune limite
	** @param (optional) Size minSize	: la taille minimale de l'objet
	** @param (optional) Size maxSize	: la taille maximale de l'objet
	** @return Integer : le nombre d'objets dtects
	*/
	unsigned int						detect(unsigned int objLimit = 0, const cv::Size& minSize = cv::Size(200, 200), const cv::Size& maxSize = cv::Size());
	/*!
	** \brief
	** Renvoie toutes les positions de l'objet dans l'image
	** @return Vector<Rect> : les rgions d'intrts
	*/
	const std::vector<cv::Rect>&		getROI() const;

protected:
	std::vector<cv::Rect>				mROI;
	cv::Mat								mInputFrame;

private:
	std::string							mClassifierName;
	cv::CascadeClassifier				mClassifier;
};

}

#endif // !OBJECTDETECT_H