/* IOT Copyright (C) 2010 CEG development team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Authors: CEG <ceg@ionlythink.com>, http://www.ionlythink.com
 */

#ifndef MAINCONTROLLER_H
#define MAINCONTROLLER_H

#include <QList>
#include <QThread>
#include "Profil.h"
#include "vrpn_Button.h"
#include "qtimer.h"
#include "IAction.h"
#include "ExecProcessAction.h"
#include "View.h"
#include "BoxController.h"
#include "BoxType.h"
#include "Window.h"

#include "eyetrack.h"
#include "gui.h"
#include "eyes_actions_callback.h"
#include "eyes_actions_callback_func.h"

#ifndef _WIN32
# include "XAction.h"
#endif // !_WIN32

class AbstractScene;
class QDir;
class ICommunicationGraphicalServer;
class IAction;
class Systray;

using namespace std;

// ThreadEx class pour l'eye-tracking, JEREM
class QThreadEx : public QThread
{
 protected:
  void run() { exec(); }
};

namespace Ceg
{
    class Window;
}
/*! \class MainController
   * \brief Classe de Controle
   *
   *  Elle Gere le fonctionnement general du CEG et ses evenements (actions | reseau)
   */
class MainController : public QObject
{
    Q_OBJECT

private:
	/*!
     *  \brief Initialise tout les paremetres pour la navigation
     */
	void							initialize();

    View							_view; /*!<La vue pour representer les scene (Une vue pour toutes les scenes) */ 
    QList<AbstractScene *>			_scenes; /*!<  L'ensemble des scenes, une scnene par fenetre systemes ouvertes + eventuellement un menu */ 
    BoxController					_boxController; /*!< Controlleur de box */ 
    ICommunicationGraphicalServer	*_comGs; /*!< ICommunicationGraphicalServer */ 
    Ceg::Window						_ownWindow; /*!< Correspond a la fenetre de notre propre programme */  
    vrpn_Button_Remote*				_VRPNButton;
    QTimer*							_timer;
    bool							_running;
	list<Profil>					_profils; /*!< Liste des profils */
	Profil							_currentProfil; /*!< Profil courant */

	QThreadEx*	_eyeTrackerThread;// Thread dans lequel est excut l'eyeTracking

	EyeTrack*	_eyeTracker;// La classe principale de l'eyeTracking (la mainloop), excute dans le _eyeTrackerThread

	eyes_actions_callback	_eyesActions;// La classe de gestion des actions envoyes depuis le _eyeTracker (possde les signaux d'envoi des actions,  relier o l'on veut)

	eyes_actions_callback_func*_eyesActionsExec;// La classe qui execute les actions correspondant aux signaux

	Gui*	_eyeTrackerGUI;// La GUI de l'eyeTracker, 4 petites fentres  updater en fontions des signaux rcuprs du _eyeTracker
	bool	_isInTrainingProcess;

public:
#ifndef _WIN32
    XAction				_xAction;
#endif //!_WIN32
	/*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe MainController
     *
     *  \param Systray : pointeur sur la classe Systray
     */
    MainController(Systray &);
	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe Systray
     */
    ~MainController();
	/*!
     *  \brief Getter pour BoxController
     */
    BoxController const &			getBoxController() const;
	/*!
     *  \brief Getter pour ICommunicationGraphicalServer
     */
    ICommunicationGraphicalServer*	getComGs() const;
	/*!
     *  \brief Getter pour la view
     */
	View &							getView();
	/*!
     *  \brief Getter pour la fenetre
     */
    Ceg::Window &					getWindow();
	/*!
     *  \brief Getter : Renvoie la scene qui est en premier dans la liste
     */
    AbstractScene*					getCurrentScene() const;
	/*!
     *  \brief Getter : renvoie la scene a la posistion
     */
    AbstractScene*					getSceneAt(int position) const;
	/*!
     *  \brief Getter : renvoie la scene qui a l'id
     */
    AbstractScene*					getScene(QString const & id) const;
	/*!
     *  \brief Getter : Retourne l'index dans la liste des scenes
     */
    int								getIndexOfScene(AbstractScene * sceneSearch) const;
	/*!
     *  \brief Getter : Retourne l'index du premier type
     */
    int								getFirstOfType(BoxType type) const;
	/*!
     *  \brief Getter : Retourne  la premiere scene de navigation : Custom ou default
     */
    AbstractScene*					getFirstNavigationScene() const;
	/*!
     *  \brief Getter : Retourne  le profil associe au nom en parametre
     */
	const Profil&					getProfil(const QString& name) const;
	/*!
     *  \brief Getter : Retourne  la liste des Profils
     */
	list<Profil>&					getProfils();

	const Profil&					getCurrentProfil();


	/*!
     *  \brief Setter : Set la liste des Profils
     */
	void							setProfils(list<Profil>&);
	/*!
     *  \brief Setter : Set un profil
     */
	void							setProfil(const Profil&);
	/*!
     *  \brief Setter : Set le profil courant
     */
	void							setCurrentProfil(Profil);
	/*!
     *  \brief load les Profils XML dans la liste
     */
	void							loadXMLProfils(QString const & fileName);
	/*!
     *  \brief initialize les profils XML
     */
	void							initializeXMLProfils();
	/*!
     *  \brief creer une nouvelle Scene
     */
    AbstractScene *					createScene(Ceg::Window const & window);
	/*!
     *  \brief Rajoute la scene au debut des scenes et rafraichit la vue
     */
    void							pushFrontScene(AbstractScene * scene);
	/*!
     *  \brief Supprime la scene du debut et rafraichit la vue
     */
    void							popFrontScene();
	/*!
     *  \brief Supprime la scene qui est a l'index et affiche le menu home
     */
    void							removeSceneAt(int index);
	/*!
     *  \brief Callback pour le casque EEG
     */
    void static VRPN_CALLBACK		vrpn_button_callback(void* user_data, vrpn_BUTTONCB button);

	QString							_sharePath; /*!< Path du dossier Share */
	QList<Box const *>				_homeMenu;
	QMap<QString, QString>			_appListing;
	bool	isInit;

public slots:
	/*!
     *  \brief Active la navigation
     */
    void							on_start_navigation();
	/*!
     *  \brief Stop la navigation
     */
    void							on_stop_navigation();
	/*!
     *  \brief Attrape les actions a execute
     */
    void							on_action_emitted(IAction &);

    /*!
     *  \brief Dmarre l'utilisation de l'eyeTracking
     */
    void	on_start_eyetracking();
    /*!
     *  \brief Arrte l'utilisation de l'eyeTracking
     */
    void	on_stop_eyetracking();

    private slots:
    void	Slt_LookLeft();
    void	Slt_LookRight();
    void	Slt_LookTop();
    void	Slt_LookBottom();
    void	Slt_LookUnknown();
    void	Slt_BlinkLeft();
    void	Slt_BlinkRight();
    void	Slt_BlinkDouble();
    void	Slt_DisplayHelp(bool);
    void	cleanup_eyetracking();
    void     	getVRPNData();
};

#endif // MAINCONTROLLER_H
