
#include <QString>
#include "objectdetect.h"
#include "Logger.h"

using namespace eyetrack;

ObjectDetect::ObjectDetect()
{
	mROI.clear();
}

ObjectDetect::~ObjectDetect()
{

}

bool	ObjectDetect::setClassifier(const std::string& cname)
{
	this->mClassifierName = cname;
	if (!this->mClassifierName.empty())
	{
		QString info_log;

		if (!this->mClassifier.load(this->mClassifierName))
		{
			//std::cerr << "Could not load classifier cascade `" << this->mClassifierName.c_str() << '`' << std::endl;
			info_log = "Could not load classifier cascade `";
			info_log.append(this->mClassifierName.c_str());
			info_log.append('`');
			Logger::getInstance()->log(ERROR_LOG, info_log);
			return false;
		}
		else
		{
			//std::cout << this->mClassifierName.c_str() << " loaded successfully" << std::endl;
			info_log = this->mClassifierName.c_str();
			info_log += " loaded successfully";
			Logger::getInstance()->log(INFO_LOG, info_log);
		}
		return true;
	}
	std::cerr << "No classifier specified" << std::endl;
	return false;
}

void	ObjectDetect::setFrame(const cv::Mat& frameToDetectFrom)
{
	if (frameToDetectFrom.empty())
		return ;
	else
		mInputFrame = frameToDetectFrom;
}

unsigned int	ObjectDetect::detect(unsigned int objLimit, const cv::Size& minSize, const cv::Size& maxSize)
{
	this->mROI.clear();

	// Vrifie si l'image n'est pas vide
	if (mInputFrame.empty())
		return 0;
	// Ou si la taille de la zone de recherche n'est pas plus grande que l'image
	if (minSize.width >= mInputFrame.cols || minSize.height >= mInputFrame.rows
		|| maxSize.width >= mInputFrame.cols || maxSize.height >= mInputFrame.rows)
		return 0;
	// Ou si une des tailles de la zone de recherche est infrieure  0
	if (minSize.width < 0 || minSize.height < 0
		|| maxSize.width < 0 || maxSize.height < 0)
		return 0;
	
	this->mClassifier.detectMultiScale(this->mInputFrame, this->mROI, 1.1, 1, 0|CV_HAAR_SCALE_IMAGE, minSize, maxSize);
	if (objLimit != 0 && objLimit < this->mROI.size())
		this->mROI.resize(objLimit);
	return this->mROI.size();
}

const std::vector<cv::Rect>&	ObjectDetect::getROI() const
{
	return this->mROI;
}
