/* IOT Copyright (C) 2010 CEG development team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Authors: CEG <ceg@ionlythink.com>, http://www.ionlythink.com
 */

/*********************************/
#include <algorithm>
/*********************************/
#include <QtGui>
#include <QFileDialog>
#include <QSettings>
#include <QMessageBox>
#include <QColorDialog>
#include <QPainter>
#include <qlistview.h>
#include <qtreeview.h>
#include <QDomDocument>
#include <sstream>
/*********************************/
#include "MainController.h"
#include "SettingsDialog.h"
/*********************************/

SettingsDialog::SettingsDialog(MainController *mainC, QWidget *parent = 0) :
  QDialog(parent), _mainC(mainC)
{
  _languages = Languages(2);
  this->setupUi(this);
  this->on_colorOpacitySlider_valueChanged(this->colorOpacitySlider->value());
  
  this->_languages[0] = "en_US";
  this->_languages[1] = "fr_FR";
  // LOAD
  
  QSettings settings;
  
  settings.beginGroup("general");

  Languages::iterator it = std::find(_languages.begin(), _languages.end(), settings.value("language").toString());
  
  if (it != _languages.end())
    {
      this->languageComboBox->setCurrentIndex(it - _languages.begin());
    }

    this->squareNumberBox->setValue(settings.value("squareNumber").toInt());
    settings.endGroup();

    settings.beginGroup("color");
    QString color = settings.value("focus").toString();
    this->colorFocusLabel->setPalette(QPalette(color));
    this->colorFocusLabel->setAutoFillBackground(true);
    color = settings.value("blur").toString();
    this->colorBlurLabel->setPalette(QPalette(color));
    this->colorBlurLabel->setAutoFillBackground(true);
    this->colorOpacitySlider->setValue((settings.value("opacity").toDouble() * 100.));
    color = settings.value("text").toString();
    this->colorTextLabel->setPalette(QPalette(color));
    this->colorTextLabel->setAutoFillBackground(true);
    settings.endGroup();

	/*
	*	Modif Loli 
	*/

	settings.beginGroup("profiles");

	connect(this->AddProfil, SIGNAL(clicked()), this, SLOT(addProfil()));
	connect(this->RemoveProfil, SIGNAL(clicked()), this, SLOT(removeProfil()));
	connect(this->EditProfil, SIGNAL(clicked()), this, SLOT(editProfil()));
	connect(this->LoadProfil, SIGNAL(clicked()), this, SLOT(loadProfil()));
	connect(this->AddProgram, SIGNAL(clicked()), this, SLOT(addProgram()));
	connect(this->RemoveProgram, SIGNAL(clicked()), this, SLOT(removeProgram()));
	this->ProfilLabel->setAlignment(Qt::AlignCenter);
	settings.endGroup();
	this->ProfillistView->setEditTriggers(0); 
    this->_profilModel = new QStringListModel();
	list<Profil> profils = mainC->getProfils();
	list<Profil>::iterator pit = profils.begin();
	Profil	profil;
	foreach (profil, profils)
		this->_profilsName.append(profil._name);

	this->_profilModel->setStringList(this->_profilsName);
    this->ProfillistView->setModel(this->_profilModel);
	if (pit != profils.end())
	{
		this->_mainC->setCurrentProfil(*pit);
		this->label_current_profil->setText(pit->_name);
	}
	this->_programsModel = new QStandardItemModel(0, 2);
	this->programView->setAlternatingRowColors(true);
	this->_programsModel->setHeaderData(0, Qt::Horizontal, tr("Nom"));
	this->_programsModel->setHeaderData(1, Qt::Horizontal, tr("Chemin"));
	QMap<QString, QString>::iterator prit;
	for (prit = this->_mainC->_appListing.begin(); prit != this->_mainC->_appListing.end(); ++prit)
	{
		this->_programsModel->insertRow(0);
		this->_programsModel->setData(this->_programsModel->index(0, 0), prit.key());
		this->_programsModel->setData(this->_programsModel->index(0, 1), prit.value());
	}
	this->programView->setModel(this->_programsModel);
}

void SettingsDialog::loadProfil()
{
  QModelIndex currentIndex = this->ProfillistView->currentIndex();
  QString selectedProfil = currentIndex.data().toString();
  if (selectedProfil != NULL)
    {
      this->label_current_profil->setText(selectedProfil);
      this->_mainC->setCurrentProfil(this->_mainC->getProfil(selectedProfil));
    }
}

void	SettingsDialog::addProgram()
{
	if (this->ProgramName->text().isEmpty() == false && this->ProgramPath->text().isEmpty() == false)
	{
		QString pathToSearch;
#if defined(Q_OS_UNIX) || defined(Q_OS_MAC)
		pathToSearch = QCoreApplication::applicationDirPath() + "/share";
#elif defined(Q_OS_WIN)
		pathToSearch = QCoreApplication::applicationDirPath() + "/../share";
#endif
		QFile			file(pathToSearch + "/ceg/config/menus/ApplicationMenu.xml");
		QDomDocument	doc(pathToSearch + "/ceg/config/menus/ApplicationMenu.xml");

		this->_programModified = true;
		this->_programsModel->insertRow(0);
		this->_programsModel->setData(this->_programsModel->index(0, 0), this->ProgramName->text());
		this->_programsModel->setData(this->_programsModel->index(0, 1), this->ProgramPath->text());
		this->programView->setModel(this->_programsModel);
    
	
		if (file.open(QIODevice::ReadWrite) == true)
	{
		if (doc.setContent(&file) == true)
		{
			QDomElement & rootElement = doc.documentElement();
			if (rootElement.tagName() == "menu")
			{
				QTextStream ts(&file);
				std::ostringstream oss;
				
				QDomElement elem = doc.createElement("box");
				QDomElement actionElem = doc.createElement("action");
				QDomElement actionChildElem = doc.createElement("action");
				QDomElement styleElem = doc.createElement("style");
				int val = rootElement.lastChild().toElement().attribute("y").toInt();
				file.resize(0);
			    oss << val;
				std::string valStr = oss.str();
				rootElement.lastChild().toElement().setAttribute("y", val + 100);
				elem.setAttribute("type", "2");
				elem.setAttribute("x", "750");
				elem.setAttribute("y", valStr.c_str());
				elem.setAttribute("height", "80");
				elem.setAttribute("width", "240");
				actionElem.setAttribute("id", "ExecMenu");
				actionChildElem.setAttribute("id", "ExecProcess");
				actionChildElem.setAttribute("time", "5000");
				actionChildElem.setAttribute("unix-path", this->ProgramPath->text());
				actionChildElem.setAttribute("win-path", this->ProgramPath->text());
				styleElem.setAttribute("fontSize", "12");
				styleElem.setAttribute("opacity", "1");
				styleElem.setAttribute("text", this->ProgramName->text());
				styleElem.setAttribute("fr-text", this->ProgramName->text());
				styleElem.setAttribute("imagePath", ":images/software/defaut.png");
				styleElem.setAttribute("rounded", "true");
				actionElem.appendChild(actionChildElem);
				elem.appendChild(actionElem);
				elem.appendChild(styleElem);
				//rootElement.appendChild(elem);
				rootElement.insertBefore(elem, rootElement.lastChild());
				ts << doc.toString();
				file.close();
			}
		}
	}
	}
}

void	SettingsDialog::removeProgram()
{
	this->_programModified = true;
	QString pathToSearch;
#if defined(Q_OS_UNIX) || defined(Q_OS_MAC)
	pathToSearch = QCoreApplication::applicationDirPath() + "/share";
#elif defined(Q_OS_WIN)
	pathToSearch = QCoreApplication::applicationDirPath() + "/../share";
#endif
	QFile			file(pathToSearch + "/ceg/config/menus/ApplicationMenu.xml");
	QDomDocument	doc(pathToSearch + "/ceg/config/menus/ApplicationMenu.xml");

	if (file.open(QIODevice::ReadWrite) == true)
	{
		if (doc.setContent(&file) == true)
		{
			QTextStream ts(&file);
			QDomElement & rootElement = doc.documentElement();
			for (QDomNode boxNode = rootElement.firstChild(); !boxNode.isNull(); boxNode = boxNode.nextSibling())
			{
				QDomElement const & boxElement = boxNode.toElement();
				if (boxElement.isNull() == false && boxElement.tagName() == "box")
				{					
					QDomElement const & styleElement = boxElement.lastChild().toElement();
					QString name(styleElement.attribute("text").toStdString().c_str());
					if (name == this->programView->selectionModel()->selection().first().indexes().begin()->data().toString())
					{
						file.resize(0);
						rootElement.removeChild(boxNode);
						doc.appendChild(rootElement);
						ts << doc.toString();
						file.close();
					}
				}
			}
		}
	}

	QItemSelection selection(this->programView->selectionModel()->selection());
	QList<int> rows;
	int prev = -1;
	foreach(const QModelIndex & index, selection.indexes())
		rows.append(index.row());
	qSort(rows);
	for(int i = rows.count() - 1; i >= 0; i--)
	{
		int current = rows[i];
		if (current != prev)
		{
			this->_programsModel->removeRows( current, 1 );
			prev = current;
		}
	}
	this->programView->setModel(this->_programsModel);
}


void SettingsDialog::addProfil()
{
	AddProfilWindow addProfilW(this);

	if (addProfilW.exec())
	{
		// creation du filename
		QString fileName = this->_mainC->_sharePath + "/profils/";
		fileName += addProfilW._profilName->text() + ".xml";
		//  creation du profil
		Profil profil(addProfilW._profilName->text(), addProfilW._vrpnIp->text(), addProfilW._shortcuts);
		this->_mainC->setProfil(profil);
		// creation du xml du profil
		fillXMLProfil(fileName, &profil);

		// Refresh de la liste des profils
		this->_profilsName.append(addProfilW._profilName->text());
		this->_profilModel->insertRow(this->_profilModel->rowCount());
		this->_profilModel->setData(this->_profilModel->index(this->_profilModel->rowCount() - 1), QVariant(addProfilW._profilName->text()), Qt::EditRole);
	}	
}

void							SettingsDialog::editProfil()
{
	AddProfilWindow				addProfilW(this);
	QStandardItemModel*			model = addProfilW.getModel();
	QTableView*					view = addProfilW.getView();
	QModelIndex					currentIndex = this->ProfillistView->currentIndex();
	QString						selectedProfil = currentIndex.data().toString();
	Profil						profil = this->_mainC->getProfil(selectedProfil);
	QMap<int, int>::iterator	itm;

	addProfilW._profilName->setText(selectedProfil);
	addProfilW._vrpnIp->setText(profil._vrpnIp);
	for (itm = profil._shortcuts.begin(); itm != profil._shortcuts.end(); ++itm)
	{
		QList<QStandardItem*>	itemList;
		QKeySequence			sequence(itm.key());
		QStandardItem*			key = new QStandardItem(sequence.toString());
		QStandardItem*			value = new QStandardItem(addProfilW._action->itemText(itm.value()));

		addProfilW._shortcuts.insert(itm.key(), itm.value());
		itemList.push_back(value);
		itemList.push_back(key);
		model->insertRow(model->rowCount() - 1, itemList);
	}
	view->setModel(model);
	if (addProfilW.exec())
	{
		profil._name = addProfilW._profilName->text();
		profil._shortcuts = addProfilW._shortcuts;
		profil._vrpnIp = addProfilW._vrpnIp->text();
		this->removeProfilByName(selectedProfil);
		this->_mainC->setProfil(profil);
		this->fillXMLProfil(this->_mainC->_sharePath + "/profils/" + selectedProfil + ".xml", &profil);
		
	}
}

void SettingsDialog::removeProfilByName(const QString& name)
{
	QString filePath = this->_mainC->_sharePath + "/profils/";
	std::list<Profil>::iterator it;
	QFile	file(filePath + name + ".xml");

	file.remove();

	for (it = this->_mainC->getProfils().begin(); it != this->_mainC->getProfils().end(); ++it)
	{
		if (it->_name == name)
		{
			this->_mainC->getProfils().remove(*it);
			break;
		}
	}
}

void SettingsDialog::removeProfil()
{
	QString filePath = this->_mainC->_sharePath + "/profils/";
	QModelIndex currentIndex = this->ProfillistView->currentIndex();
	QString selectedProfil = currentIndex.data().toString();
	QFile	file(filePath + selectedProfil + ".xml");
	std::list<Profil>::iterator it;
	QList<QString>::iterator qit;

	for (it = this->_mainC->getProfils().begin(); it != this->_mainC->getProfils().end(); ++it)
	{
		if (it->_name == selectedProfil)
		{
			this->_mainC->getProfils().remove(*it);
			break;
		}
	}
	for (qit = this->_profilsName.begin(); qit != this->_profilsName.end(); ++qit)
	{
		if (*(qit) == selectedProfil)
		{
			qit->remove(selectedProfil);
			break;
		}
	}
	this->_profilModel->removeRow(currentIndex.row(), currentIndex.parent());
	file.remove();
	if (selectedProfil == this->_mainC->getCurrentProfil()._name)
	{
		this->label_current_profil->setText("");
	}
}

void				SettingsDialog::fillXMLProfil(QString fileName, Profil *profil)
{
	QFile			file(fileName);
	QDomDocument	doc("profil");

	if (file.open(QIODevice::ReadWrite) == true)
	{
		QTextStream						ts(&file);
		QMap<int, int>::const_iterator	it;

		QDomProcessingInstruction xmlHeaderPI = doc.createProcessingInstruction("xml", "version=\"1.0\" " );
		doc.appendChild(xmlHeaderPI);

		QDomElement						rootElement = doc.createElement("profil");

		rootElement.setAttribute("name", profil->_name);
		for (it = profil->_shortcuts.begin(); it != profil->_shortcuts.end(); it++)
		{
			QDomElement elem = doc.createElement("shortcut");

			elem.setAttribute("action", it.value());
			elem.setAttribute("key", it.key());
			rootElement.appendChild(elem);
		}
		QDomElement vrpn_elem = doc.createElement("vrpn");
		if (profil->_vrpnIp.isEmpty() == true)
			vrpn_elem.setAttribute("ip", "localhost");
		else
			vrpn_elem.setAttribute("ip", profil->_vrpnIp);
		rootElement.appendChild(vrpn_elem);
		doc.appendChild(rootElement);
		ts << doc.toString();
		file.close();
	}
}

void SettingsDialog::on_confList_currentItemChanged(QListWidgetItem* current, QListWidgetItem* previous)
{
    if (!current)
	current = previous;

    this->confStack->setCurrentIndex(this->confList->row(current));
}

void SettingsDialog::on_buttonBox_accepted()
{
    QSettings settings;

    settings.beginGroup("general");

    if (this->languageComboBox->currentIndex() < _languages.size() )
    {
        QString language = _languages[ this->languageComboBox->currentIndex() ];

        if (settings.value("language").toString() != language)
        {
            QMessageBox::information
                    (0,
                     tr("Language changed"),
                     tr("The language has been changed. To apply this modification you have to restart the program.")
                     );

            settings.setValue("language", language);
        }
    }

    settings.setValue("squareNumber", this->squareNumberBox->text());
    settings.endGroup();

    settings.beginGroup("color");

    QString focusColor( this->colorFocusLabel->palette().color(QPalette::Background).name() );
    QString blurColor( this->colorBlurLabel->palette().color(QPalette::Background).name() );
    double opacityValue = this->colorOpacitySlider->value() / 100.;
    QString textColor( this->colorTextLabel->palette().color(QPalette::Background).name() );

    if (settings.value("focus").toString() != focusColor ||
	settings.value("blur").toString() != blurColor ||
	settings.value("opacity").toDouble() != opacityValue ||
	settings.value("text").toString() != textColor)
    {
        QMessageBox::information
		(0,
		 tr("Color changed"),
		 tr("To apply correctly colors updates, we recommand you to restart the program.")
		 );

        settings.setValue("focus", focusColor);
        settings.setValue("blur", blurColor);
        settings.setValue("opacity", opacityValue);
        settings.setValue("text", textColor);
    }
    settings.endGroup();

    this->accept();
}

void SettingsDialog::on_buttonBox_rejected()
{
    this->reject();
}

void SettingsDialog::on_colorFocusButton_clicked()
{
    QColor color = QColorDialog::getColor(this->colorFocusLabel->palette().color(QPalette::Background), this);
    if (color.isValid()) {
	this->colorFocusLabel->setPalette(QPalette(color));
    }
}

void SettingsDialog::on_colorBlurButton_clicked()
{
    QColor color = QColorDialog::getColor(this->colorBlurLabel->palette().color(QPalette::Background), this);
    if (color.isValid()) {
	this->colorBlurLabel->setPalette(QPalette(color));
    }
}

void SettingsDialog::on_colorOpacitySlider_valueChanged(int value)
{
    qreal realValue = value;
    realValue /= 100;
    QColor color(Qt::blue);
    color.setAlphaF(realValue);
    this->colorOpacityLabel->setPalette(QPalette(color));
    this->colorOpacityLabel->setAutoFillBackground(true);
}

void SettingsDialog::on_colorTextButton_clicked()
{
    QColor color = QColorDialog::getColor(this->colorTextLabel->palette().color(QPalette::Background), this);
    if (color.isValid()) {
	this->colorTextLabel->setPalette(QPalette(color));
    }
}
